; ------------------------------------
;  collisions.scm - Ben VandenBos
;
;  *Functions for detection collision
;   between the client's tank and any
;   other objects in the field.
; ------------------------------------

(define *collision-detection-delay* 0.010)
(define *bullet-hit-threshold* (+ (/ *tank-height-width* 2) *bullet-width*)) ;1/2(tank_width) + bullet_width
(define *tank-hit-threshold* *tank-height-width*)

; Requires the game-state semaphore
(define detect-collisions
  (lambda ()
    (semaphore-wait *game-state-semaphore*)
    (let ((client (vector-ref *game-state* *you-are-index*)))
      (semaphore-post *game-state-semaphore*)
      (letrec ((pos (exact-position client))
               (client-x (car pos))
               (client-y (cadr pos)))
        (set! client-x (+ client-x (/ *tank-height-width* 2)))
        (set! client-y (+ client-y (/ *tank-height-width* 2)))
        (let loop ((index 0))
          ; If you're not checking for a collision with yourself
          (if (not (= index *you-are-index*))
              (begin
                (semaphore-wait *game-state-semaphore*)
                (let ((obj (vector-ref *game-state* index)))
                  (semaphore-post *game-state-semaphore*)
                  ; If it's a valid object...
                  (if (object? obj)
                      ; If it's not an object owned by this client
                      (if (not (equal? (object-owner obj) *you-are-index*))
                          (letrec ((obj-pos (exact-position obj))
                                   (obj-x (car obj-pos))
                                   (obj-y (cadr obj-pos))
                                   (threshold 0))
                            (cond
                              ((equal? 'tank (object-type obj))
                               (set! obj-x (+ obj-x (/ *tank-height-width* 2)))
                               (set! obj-y (+ obj-y (/ *tank-height-width* 2)))
                               (set! threshold *tank-hit-threshold*))
                              ((equal? 'bullet (object-type obj))
                               (set! obj-x (+ obj-x (/ *bullet-width* 2)))
                               (set! obj-y (+ obj-y (/ *bullet-width* 2)))
                               (set! threshold *bullet-hit-threshold*))
                              )
                            (if (close-enough client-x client-y obj-x obj-y threshold)
                                (begin
                                  (display "Collision detected, killing collision thread...[detect-collisions]\n")
                                  (disp-console "You were killed")
                                  (collision index)
                                  (kill-thread (current-thread))))
                            )
                         )
                      )
                  )
                )
              )
          (if (< index (- (vector-length *game-state*) 1))
              (loop (+ index 1)))
          )
        (sleep/yield *collision-detection-delay*)
        (detect-collisions)
        )
      )
    )
  )

(define close-enough
  (lambda (x1 y1 x2 y2 threshold)
    (if (and (< (abs (- x1 x2)) threshold)
             (< (abs (- y1 y2)) threshold))
        #t
        #f)))