; -------------------------
;  gui.scm - Ben VandenBos
;
;  *GUI definition
; -------------------------


; GUI Definition and Console Functions
; ------------------------------------

; display a string in the console
(define disp-console
  (lambda (str)
    (send console set-value (string-append (send console get-value) str "\n"))
    ))

; callback function for console input
(define console-input-callback
  (lambda (textfield eventcontrol)
      (cond
        ((eqv? (send eventcontrol get-event-type) 'text-field-enter)
         (send-chat (string-append *nickname* ": " (send textfield get-value)))
         (send textfield set-value "")))))

; callback for hitting enter to connect
(define connect-textfield-callback
  (lambda (textfield eventcontrol)
      (cond
        ((eqv? (send eventcontrol get-event-type) 'text-field-enter)
         (go-connect)))))
; hide/show console
(define toggle-console-view
  (lambda (item event)
    (if (send console-frame is-shown?)
        (begin
          (send console-frame show #f)
          (send view-console set-label "Show Console"))
        (begin
          (send console-frame show #t)
          (send main-frame show #t)
          (send view-console set-label "Hide Console")))))

; sprites/wire-frame display
(define toggle-sprites
  (lambda (item event)
    (if *wire-frame?*
        (begin
          (set! *wire-frame?* #f)
          (send view-sprites set-label "Use Wire-Frames")
          (send mem-dc draw-bitmap game-board 0 0))
        (begin
          (set! *wire-frame?* #t)
          (send view-sprites set-label "Use Sprites")
          (send mem-wire-dc set-brush *white-brush*)
          (send mem-wire-dc set-pen *no-pen*)
          (send mem-wire-dc draw-rectangle 0 0 2000 1600))
        )
    (send drawspace on-paint)
    ))

; disconnect callback function
(define disconnect-callback
  (lambda ()
    ; close connection
    (close-output-port *tcp-out*)

    (set! *playing* #f)
    
    ; set menu items
    (send file-connect enable #t)
    (send file-disconnect enable #f)
    
    ; kill threads, clear game-state
    (clear-game-state)
    (thread-suspend *refresh-thread*)
  
    ; clear the canvas
    (send mem-dc draw-bitmap game-board 0 0)
    (send mem-wire-dc set-brush *white-brush*)
    (send mem-wire-dc set-pen *no-pen*)
    (send mem-wire-dc draw-rectangle 0 0 2000 1600)
    (send drawspace on-paint)
    
    (disp-console (string-append "Disconnected from " *server-address*))
    ))

; exit function
(define exit-callback
  (lambda (item event)
    ; close connect
    (if (output-port? *tcp-out*)
        (close-output-port *tcp-out*))
    
    (set! *playing* #f)
    
    ; kills all threads
    (clear-game-state) 
    (kill-thread *refresh-thread*)
    
    (exit)
    ))

; Main-Frame
(define main-frame%
  (class frame%
    (override on-subwindow-char on-close on-move)
    (define on-subwindow-char 
      (lambda (canvas key-event) #f))
    (define on-close
      (lambda ()
        (exit)))
    (define on-move
      (lambda (x y)
        (send console-frame move (+ x *width*) y)))
    (super-instantiate ())))

(define main-frame (instantiate main-frame% ("blue" #f *width* *height* 0 0 '(no-resize-border) #t 0 0 '(center top) 0 0 #t #t)))
(define menubar (instantiate menu-bar% (main-frame (lambda (event) ()))))
(define file-menu (instantiate menu% ("File" menubar "File Menu" (lambda (event) ()))))
(define view-menu (instantiate menu% ("View" menubar "View Settings" (lambda (event) ()))))
(define file-connect
  (instantiate menu-item% ("Connect" file-menu 
                           (lambda (item event)
                             (send connect-dialog move (+ (send main-frame get-x) 100) (+ (send main-frame get-y) 100))
                             (send connect-dialog show #t)))))
(define file-disconnect
  (instantiate menu-item% ("Disconnect" file-menu (lambda (item event) (disconnect-callback)) #f)))
(send file-disconnect enable #f)
(instantiate separator-menu-item% (file-menu))
(define file-exit
  (instantiate menu-item% ("Exit" file-menu exit-callback)))
(define my-canvas%
  (class canvas%
    (override on-event on-char on-paint)
    (define on-event (lambda (mouse-event) ()))
    (define on-char
      (lambda (key-event)
        (handle-controls (send key-event get-key-code) (send key-event get-key-release-code))))
    (define on-paint
      (lambda ()
        (if *wire-frame?*
            (send dc draw-bitmap mem-wire-bitmap (* -1 *x-offset*) (* -1 *y-offset*))
            (send dc draw-bitmap mem-bitmap (* -1 *x-offset*) (* -1 *y-offset*)))))
    (super-instantiate ())))

; Common Objects
(define *no-pen* (instantiate pen% ("BLACK" 1 'transparent)))
(define *no-brush* (instantiate brush% ("BLACK" 'transparent)))
(define *white-brush* (instantiate brush% ("WHITE" 'solid)))
(define drawspace (instantiate my-canvas% (main-frame)))
(define dc (send drawspace get-dc))

; Console Frame class
(define console-frame%
  (class frame%
    (override on-close)
    (define on-close
      (lambda ()
        (send view-console set-label "Show Console")))
    (super-instantiate ())))

; Double Buffering
; ----------------
(define mem-bitmap (instantiate bitmap% (2000 1600 #f)))
(define mem-wire-bitmap (instantiate bitmap% (2000 1600 #f)))
(define mem-dc (instantiate bitmap-dc% (mem-bitmap)))
(define mem-wire-dc (instantiate bitmap-dc% (mem-wire-bitmap)))
(send mem-dc draw-bitmap game-board 0 0)
(send mem-wire-dc set-brush *white-brush*)
(send mem-wire-dc set-pen *no-pen*)
(send mem-wire-dc draw-rectangle 0 0 2000 1600)

; Console-Frame
(define console-frame (instantiate console-frame% ("Console" main-frame 300 *height* *width* 0)))
(define console (instantiate text-field% (#f console-frame (lambda (x y) ()) "" '(multiple) #f 0 0 300 100 #t #t)))
(define console-input (instantiate text-field% (#f console-frame console-input-callback "" '(single) #t 1 1 300 4 #t #f)))
(define view-console (instantiate menu-item% ("Hide Console" view-menu toggle-console-view))) 
(define default-sprite-text (if *wire-frame?* 
                                "Use Sprites"
                                "Use Wire-Frames"))
(define view-sprites (instantiate menu-item% (default-sprite-text view-menu toggle-sprites)))

; dialog connect button callback
(define connect-dialog-connect-callback
  (lambda (b e)
    (go-connect)))

; connect
(define go-connect
  (lambda ()
    (set! *nickname* (send connect-dialog-nick get-value))
    (if (null? *nickname*)
        (set! *nickname* "Player"))
    (set! *server-address* (send connect-dialog-server get-value))
    (set! *server-port* (send connect-dialog-port get-value))
    (thread (lambda () (connect *server-address* *server-port*)))
    (thread-resume *refresh-thread*)
    (send connect-dialog show #f)))

; Connect Dialog
(define connect-dialog (instantiate dialog% ("Connect" main-frame 300 500 100 100 null #t 0 0 '(center top) 0 0 #f #f)))
(define connect-dialog-edit-panel (instantiate vertical-panel% (connect-dialog null #t 15 15 0 0 '(left center) 0 0 #t #f)))
(define connect-dialog-button-panel (instantiate horizontal-panel% (connect-dialog null #t 15 0 0 0 '(center center) 0 0 #t #f)))
(define connect-dialog-nick (instantiate text-field% ("Nick      " connect-dialog-edit-panel connect-textfield-callback *nickname* '(single) #t 0 2 160 1 #f #f)))
(define connect-dialog-server (instantiate text-field% ("Server   " connect-dialog-edit-panel connect-textfield-callback *server-address* '(single) #t 0 2 160 1 #f #f)))
(define connect-dialog-port (instantiate text-field% ("Port       " connect-dialog-edit-panel connect-textfield-callback *server-port* '(single) #t 0 2 50 1 #f #f)))
(define connect-dialog-connect (instantiate button% ("Connect" connect-dialog-button-panel connect-dialog-connect-callback null #t 0 0 20 20 #f #f)))
(define connect-dialog-cancel (instantiate button% ("Cancel" connect-dialog-button-panel (lambda (b e) (send connect-dialog show #f)) null #t 0 0 20 20 #f #f)))

; Display the main frame and console frame
(send console-frame show #t)
(send main-frame show #t)

; Double Buffering thread
; -----------------------
(define refresh (instantiate timer% ((lambda () (send drawspace on-paint)) 20)))
(define *refresh-thread* (thread (lambda () (send refresh start 20 #f))))
(thread-suspend *refresh-thread*)