;;; 'blue' server collision-handling functions
;;; Justin R. Hall (c) 2003

; this is called when a client detects a collision
(define collision
  (lambda (index2 in out)
    (let ((index1 #t)
          (accomplice-obj (vector-ref *game-state* index2))
          (accomplice-in '())
          (accomplice-out '()))
      (begin
        (set! index1 (hash-table-get *obj-assoc* out (lambda () (#f))))
        (if index1
            ; index1 is now the index into the game-state of whoever claimed
            ; to have witnessed a collision (with index2)
            (begin
              ; remove the bullet, if it was a bullet
              (if (and (object? accomplice-obj)
                       (eqv? (eval (object-type accomplice-obj)) 'bullet))
                  (call (remove-bullet *game-state* index2)))
              
              ; tell everyone about removing the tank
              ; and actually remove it
              (display `(remove-object ,index1) out)
              (call (remove-assoc-tank *game-state* out))

              ; wait for a while...
              (sleep *respawn-time*)

              ; create a new tank for whoever sent the collision message
              (let* ((next-index (call (next-available-index *game-state*)))
                     (object-params
                      `(,next-index 'tank ,(+ (random *field-width*) 100) ,(+ (random *field-height*) 100)
                        0 'stop 'stop ',(get-random-color) ,next-index)))
                
                ; put the new tank in the master game-state
                (if *debug*
                    (begin
                      (display "Adding replacement tank to master *game-state*...")
                      (newline)))
                (call (add-object (append object-params `(,in ,out))))
                
                (if *debug*
                    (begin
                      (display "*game-state*: ")
                      (display *game-state*)
                      (newline)))
                
                ; tell everyone about the new tank
                (call (tell-all-except (cons 'add-object object-params) #f))
                
                ; tell the client which tank he is
                (display `(you-are ,next-index) out)))
            (display "error: could not find tank in game-state"))))))

